#!/usr/bin/perl
################################################################################
# perlQuestionnaire                          Version 2.2                       #
# Copyright 2003-2005 chumpsoft, inc.        March 28, 2005                    #
################################################################################
#              SCRiPTMAFiA 2005 - THE DiRTY HANDS ON YOUR SCRiPTS              #
################################################################################

#-->> Push correct path into @INC depending on whether this is Unix/Windows.
if ($0 =~ /(.*)\\[^\\]+/) { unshift (@INC, "$1\\perlQMod"); }
elsif ($0 =~ /(.*)\/[^\/]+/) { unshift (@INC, "$1/perlQMod"); }
else { unshift (@INC, "perlQMod"); }

#-->> Include Files

use strict;
require "State.cgi";
import perlQMod::State;
require "Question.cgi";
require "View.cgi";

my $perlQ = perlQMod::State->new;
my $cgi = $perlQ->getCGI;
my $dbh = $perlQ->getDBH;
my $html = $perlQ->getHTML;

#-->> Setting Variables

my $action = $cgi->param('action') || '';
my $sid = $cgi->param('sid') || 0;
my $cq = $cgi->param('cq') || 1;

$sid = ($sid =~ /^(\d+)$/) ? $1 : 0;
$cq = ($cq =~ /^(done|\d+)$/) ? $1 : 1;

#-->> Admin Preview Only

my $admin = 0;
if ($action eq "preview") {
	if (!$perlQ->isAdmin) {
		$html->redirect(-url => 'error.cgi');
	}
	$admin = 1;
}

#-->> Cookie Test

$html->setcookie(-name => 'perlQTest', -value => '1');

#-->> Retrieving specified survey.

my $masurvey = 0;
my $query = $dbh->prepare("SELECT *, UNIX_TIMESTAMP(StartDate) as starttime, UNIX_TIMESTAMP(EndDate) as endtime, UNIX_TIMESTAMP(NOW()) as nowtime FROM perlQSurvey WHERE SID=?");
if ($query->execute($sid) && $query->rows) {
	$masurvey = $query->fetchrow_hashref;
}
$query->finish;

if (!$masurvey) {
	if ($perlQ->getAdminField('Template')) {
		$html->redirect(-url => 'index.cgi');
	}
	else {
		$html->redirect(-url => 'error.cgi');
	}
}

if ($masurvey->{'SurveyResults'} == 2 && (defined $cgi->param('results') || defined $cgi->param('results.x'))) {
	$html->redirect(-url => 'stats.cgi?sid='.$html->protectHTML($masurvey->{'SID'}));
}

#-->> Set Language...

$perlQ->setLanguage($masurvey->{'Language'});
my $lang = $perlQ->getLanguage;

my ($errormsg,$contents) = ('','');

#-->> Checking Activation

if ($masurvey->{'Active'} eq 'n' && $action ne "preview") {
	$errormsg = sprintf($lang->{'SurveyNotActive'}, $masurvey->{'Name'});
}
elsif ($action ne "preview") {
	if (substr($masurvey->{'StartDate'},0,10) ne '0000-00-00' &&
	    $masurvey->{'starttime'} > $masurvey->{'nowtime'}) {
		$errormsg = sprintf($lang->{'SurveyPending'}, $masurvey->{'Name'});
	}
	elsif (substr($masurvey->{'EndDate'},0,10) ne '0000-00-00' &&
	    $masurvey->{'endtime'} < $masurvey->{'nowtime'}) {
		$errormsg = sprintf($lang->{'SurveyExpired'}, $masurvey->{'Name'});
	}
}

#-->> Question Counting

my $surveyqcount = 0;
$query = $dbh->prepare("SELECT MAX(SortOrder) FROM perlQQuestion WHERE SID=?");
if ($query->execute($sid) && $query->rows) {
	$surveyqcount = ($query->fetchrow_array)[0];
}
$query->finish;

#-->> Checking for Survey Completion

my @questionresult = &getQuestions($perlQ, $cq, $surveyqcount, $masurvey);
if ($cq eq 'done' && $questionresult[1] eq 'complete' && $action ne 'preview') {

	if ($surveyqcount > 0) {
		my $storesurvey = 1;

		# Different Methods of Marking Completion

		my $storeddata;

		my $ufid = $cgi->param('ufid') || '';
		if ($ufid ne '') {
			$query = $dbh->prepare("SELECT UID,Email,EmailConfirm FROM perlQUser WHERE BlockDupe=? AND SID=? LIMIT 1");
			if ($query->execute($ufid, $sid) && $query->rows) {
				$storeddata = $query->fetchrow_hashref;
				$storesurvey = 0;
			}
		}
		else {
			$storesurvey = 0;
		}

		if (substr($masurvey->{'SurveyUsers'}, 0, 1)) {	# Unique IP
			$query = $dbh->prepare("SELECT 1 FROM perlQUser WHERE IPAddr=? AND SID=? LIMIT 1");
			if ($query->execute($ENV{'REMOTE_ADDR'}, $sid) && $query->rows) {
				$storesurvey = 0;
			}
			$query->finish;
		}

		my $storereason = '';
		my $cookiename = "perlQ".$perlQ->getAdminField('perlQID')."_$sid";
		if (substr($masurvey->{'SurveyUsers'}, 1, 1)) {	# Cookies
			my $cuid = $cgi->cookie($cookiename) || '';
			if ($cuid ne '' || !$cgi->cookie('perlQTest')) {
				$storesurvey = 0;
				if ($cuid ne '' && $cuid =~ /^\d+$/) {
					$query = $dbh->prepare("SELECT UID,Email,EmailConfirm FROM perlQUser WHERE SID=? AND UID=? LIMIT 1");
					if ($query->execute($sid, $cuid) && $query->rows) {
						$storeddata = $query->fetchrow_hashref;
					}
				}
				else { $storereason = 'cookies'; }
			}
		}

		my $verify = 0;
		if (substr($masurvey->{'SurveyUsers'}, 2, 1) && # Verified Email
		    ($perlQ->getAdminField('SendmailPath') || $perlQ->getAdminField('SMTPServer'))) {
			$verify = 1;
		}

		if ($storesurvey) {
			my $uid = $perlQ->newUID;

			if (substr($masurvey->{'SurveyUsers'}, 1, 1)) { # Set Cookie Restriction
				$html->setcookie(-name => $cookiename, -value => $uid, -expires => '+10y', -path => '/');
			}

			eval {
				$dbh->do("INSERT INTO perlQUser (UID, SID, IPAddr, BlockDupe, CheckType, CompleteDate, Confirm) VALUES (?, ?, ?, ?, ?, NOW(), ?)", undef, $uid, $sid, $ENV{'REMOTE_ADDR'}, $ufid, $masurvey->{'SurveyUsers'}, int(rand(899999) + 100000));
			};
		
			$query = $dbh->prepare("SELECT * FROM perlQQuestion WHERE SID=? AND Type!='htmlcontent' ORDER BY SortOrder");
			if ($query->execute($sid)) {
				for (my $i = 0; $i < $query->rows; $i++) {
					my $maqlist = $query->fetchrow_hashref;
					if ($maqlist->{'Type'} eq 'pagebreak') { next; }
					my ($valid, $answer) = &storeAnswer($perlQ, $maqlist, $masurvey);
					if ($valid) {
						eval {
							$dbh->do("INSERT INTO perlQAnswer (UID, SID, QID, Answer) VALUES (?, ?, ?, ?)", undef, $uid, $sid, $maqlist->{'QID'}, $answer);
						};
					}
				}
			}
			$query->finish;

			if ($masurvey->{'ResponseNotice'} && $masurvey->{'ResponseEmail'} ne '') {
				my $surveydata = '';
				my $headers = '';
				if ($masurvey->{'ResponseNotice'} == 2) {
					$surveydata = &recreateSurvey($perlQ, $sid, $uid, 0, $masurvey, 1);
					$headers = "Content-type: text/html\n";
				}
				else {
					my $scriptloc = $cgi->url;
					if ($scriptloc =~ m#^(.*)/[^/]+$#) { $scriptloc = $1; }
					$scriptloc = "$scriptloc/view.cgi?sid=$sid&uid=$uid";

					$surveydata = $lang->{'RNEmailLinkText'};
					$surveydata =~ s/::SurveyName::/$masurvey->{'Name'}/g;
					$surveydata =~ s/::ResultsLink::/$scriptloc/g;
					my $query = $dbh->prepare("SELECT * FROM perlQUser WHERE SID=? AND UID=?");
					if ($query->execute($sid, $uid) && $query->rows) {
						my $mainfo = $query->fetchrow_hashref;
						$surveydata =~ s/::CompleteDate::/$mainfo->{'CompleteDate'}/g;
						$surveydata =~ s/::IPAddress::/$mainfo->{'IPAddr'}/g;
					}
					$query->finish;
				}

				$perlQ->sendEmail(
					-from => $masurvey->{'EmailFrom'} || $lang->{'EmailDefaultFrom'},
					-to => $masurvey->{'ResponseEmail'}, 
					-subject => sprintf($lang->{'ViewSurvey'}, $masurvey->{'Name'}),
					-body => $surveydata,
					-headers => $headers);
			}

			if ($verify) {
				$html->redirect(-url => "verify.cgi?uid=$uid&sid=$sid");
			}

			# Complete Notice

			if ($masurvey->{'OnComplete'} == 1 && $masurvey->{'SurveyResults'}) {
				$html->redirect(-url => "stats.cgi?sid=$sid");
			}
			elsif ($masurvey->{'OnComplete'} == 2 && $masurvey->{'RedirectURL'}) {
				$html->redirect(-url => $html->protectHTML($masurvey->{'RedirectURL'}));
			}
			elsif ($masurvey->{'OnComplete'} == 3) {
				$contents = $lang->{'YourResponse'}.'<BR><BR>';
				$contents .= &recreateSurvey($perlQ, $sid, $uid, 0, $masurvey, 1);
			}
			elsif ($masurvey->{'Completed'}) { $contents = $masurvey->{'Completed'}; }
			else {
				my $defmsg = sprintf($lang->{'SurveyCompleted'}, $masurvey->{'Name'});
				$contents = $html->printMessage($defmsg, $masurvey->{'Width'});
			}
		}
		else {
			if ($storereason eq 'cookies') {
				$errormsg = sprintf($lang->{'SurveyCookieError'}, $masurvey->{'Name'});
			}

			# If the storage failed for another reason we will try to recover to the
			# the correct stage of the survey.
			else {
				if ($storeddata) {
					my $uid = $storeddata->{'UID'};

					# Verified Email
					if ($verify) {
						if ($storeddata->{'Email'} eq '') {
							$html->redirect(-url => "verify.cgi?uid=$uid&sid=$sid");
						}
						elsif ($storeddata->{'EmailConfirm'} eq 'n') {
							$html->redirect(-url => "confirm.cgi?uid=$uid&sid=$sid");
						}
					}

					# Complete Notice
					if ($masurvey->{'OnComplete'} == 1 && $masurvey->{'SurveyResults'}) {
						$html->redirect(-url => "stats.cgi?sid=$sid");
					}
					elsif ($masurvey->{'OnComplete'} == 2 && $masurvey->{'RedirectURL'}) {
						$html->redirect(-url => $html->protectHTML($masurvey->{'RedirectURL'}));
					}
					elsif ($masurvey->{'OnComplete'} == 3) {
						$contents = $lang->{'YourResponse'}.'<BR><BR>';
						$contents .= &recreateSurvey($perlQ, $sid, $uid, 0, $masurvey, 1);
					}
					elsif ($masurvey->{'Completed'}) { $contents = $masurvey->{'Completed'}; }
					else {
						$errormsg = sprintf($lang->{'SurveyAlreadyCompleted'}, $masurvey->{'Name'});
					}
				}
				else {
					$errormsg = sprintf($lang->{'SurveyCompletedAlready'}, $masurvey->{'Name'});
				}
			}
		}
	}
}
elsif ($cq eq 'done' && $questionresult[1] eq 'complete' && $action eq 'preview') {
	my $prevcompmsg = sprintf($lang->{'SurveyPreviewCompleted'}, $masurvey->{'Name'});
	$contents = "<TABLE CELLPADDING=0 CELLSPACING=0 BORDER=0 WIDTH=100%>\n";
	$contents .= "<FORM>\n";
	$contents .= "<TR><TD ALIGN=center STYLE=\"".$masurvey->{'ErrorStyle'}."\">";
	$contents .= "$prevcompmsg</TD></TR>\n";
	$contents .= "<TR><TD>$masurvey->{'Completed'}</TD></TR>\n";;
	$contents .= "<TR><TD ALIGN=center><INPUT TYPE=button ONCLICK=\"window.close();\" VALUE=\"$lang->{'Close'}\"></TD></TR>\n";
	$contents .= "</FORM>\n";
	$contents .= "</TABLE>\n";
}
else {
	$contents .= $html->createSurvey($masurvey, $cq, $questionresult[2], $admin);
}

#-->> Checking for Questions in Survey

if ($surveyqcount == 0) {
	$errormsg = sprintf($lang->{'SurveyEmpty'}, $masurvey->{'Name'});
}

#-->> Overwriting Contents if Deactivated

$contents = ($errormsg) ? $html->printMessage($errormsg, $masurvey->{'Width'}) : $contents;

#-->> Output Page

$perlQ->getHTML->createPage($sid, $masurvey->{'Name'}, $contents, 0, '');
