#!/usr/bin/perl
################################################################################
# perlQuestionnaire                          Version 2.2                       #
# Copyright 2003-2005 chumpsoft, inc.        March 28, 2005                    #
################################################################################
#              SCRiPTMAFiA 2005 - THE DiRTY HANDS ON YOUR SCRiPTS              #
################################################################################

use strict;

sub importSurvey {
	my ($perlQ, $content, $type, $flag) = @_;
	my $dbh = $perlQ->getDBH;
	my $html = $perlQ->getHTML;
	my $sid = 0;
	my $lang = $perlQ->getLanguage;

	my $action = ($type eq 'SWAP') ? $lang->{'ImportFailed'} : $lang->{'RestoreFailed'};
	if ($content->[0] !~ /^<!--(PERL|PHP)QBACKUP ([\d\.]+) (\1QBACKUP|SURVEYBACKUP|SURVEYSWAP)-->$/) {
		$html->setMessage("$action$lang->{'ImportBadHeader'}");
		return 0;
	}
	my ($qlang, $version, $btype) = ($1, $2, $3);

	if (lc($qlang) ne 'perl') {
		$btype =~ s/$qlang/PERL/i;
		for (my $i = 1; $i < @{ $content }; $i++) {
			$content->[$i] =~ s/$qlang/perl/i;
		}
	}

	if ($version < 1.2 || $version > $perlQ->getVersion) {
		$html->setMessage($action.sprintf($lang->{'ImportBadVersion'}, $version, $perlQ->getVersion));
		return 0;
	}

	if ($type eq 'SWAP' && $btype ne 'SURVEYSWAP') {
		$html->setMessage($action.sprintf($lang->{'ImportSurveyBadFormat'}, $html->protectHTML($btype)));
		return 0;
	}
	elsif ($type eq 'BACKUP' && $btype ne 'PERLQBACKUP' && $btype ne 'SURVEYBACKUP') {
		$html->setMessage($action.sprintf($lang->{'RestoreSurveyBadFormat'}, $html->protectHTML($btype)));
		return 0;
	}

	my (%perlQF, %perlQC, %perlQR, %perlQU, %tcreated, %idmap);
	if ($btype eq 'SURVEYSWAP') {
		$perlQF{'perlQSurvey'} = ['Name','SurveyUsers','SurveyResults','Width','Indent','QuestionNumb','QuestionStyle','AnswerInputStyle','AnswerTextStyle','AnswerStyle','ErrorStyle','VerifyEmail','EmailFrom','EmailSubject','Template','Language','OnComplete','RedirectURL','Completed','Description','AdminNotes','Active','StartDate','EndDate','AdvanceInterface','ResultsWidth','ResultsBorder','ResultsAnswer','ResultsStats','ResultsGraph','SubmitImageSrc','SubmitImageWidth','SubmitImageHeight','SubmitImageText','ResultImageSrc','ResultImageWidth','ResultImageHeight','ResultImageText','NextImageSrc','NextImageWidth','NextImageHeight','NextImageText','ResponseNotice','ResponseEmail'];
		$perlQF{'perlQQuestion'} = ['SortOrder','Question','Type','Mult','QuestionStyle','Instructions','Answer','Choose','DefaultAnswer','LargestAID','ColNumb','AnswerInputStyle','AnswerTextStyle','AnswerStyle','Attributes','Required','Summary'];
	}
	else {
		if ($btype eq 'PERLQBACKUP') {
			$perlQF{'perlQAdmin'} = ['perlQID','Version','Template','Language','AdminLanguage','SendmailPath','SMTPServer','SummaryLimit','TextareaLimit','PopupLimit','AdvanceInterface','InCgiBin','ListSurvey','TimeOut','CookieDomain'];
			$perlQC{'perlQAdmin'} = "CREATE TABLE perlQAdmin (perlQID BIGINT(20) UNSIGNED NOT NULL, Version CHAR(10) NOT NULL, Passwd VARCHAR(20) NOT NULL, Template VARCHAR(50) NOT NULL, Language VARCHAR(50) NOT NULL, AdminLanguage VARCHAR(50) NOT NULL, SendmailPath VARCHAR(100) NOT NULL, SMTPServer VARCHAR(100) NOT NULL, SummaryLimit INT UNSIGNED NOT NULL DEFAULT 10, TextareaLimit INT UNSIGNED NOT NULL DEFAULT 5, PopupLimit INT UNSIGNED NOT NULL DEFAULT 5, AdvanceInterface ENUM ('y','n') NOT NULL DEFAULT 'n', InCgiBin ENUM('y','n') NOT NULL DEFAULT 'y', ListSurvey ENUM('y','n') NOT NULL DEFAULT 'y', TimeOut BIGINT UNSIGNED NOT NULL, CookieDomain VARCHAR(255) NOT NULL)";
		}

		# perlQF is the format of our columns for each table. This ensures that no
		# extra columns that don't exist will attempt to be inserted. perlQC is the
		# create statement for a specific table. perlQR is a list of the references
		# in each table that may change and the change will be noted in $idmap.
		# perlQU is the unique field in a given table which we may have to change
		# if there is a collision.
		$perlQF{'perlQAnswer'} = ['UID','SID','QID','Answer'];
		$perlQC{'perlQAnswer'} = "CREATE TABLE perlQAnswer (UID BIGINT UNSIGNED NOT NULL, SID BIGINT UNSIGNED NOT NULL, QID BIGINT UNSIGNED NOT NULL, Answer TEXT NOT NULL, UNIQUE uniqids (SID, QID, UID), INDEX index_uid (UID))";
		$perlQR{'perlQAnswer'} = [0, 'UID', 1, 'SID', 2, 'QID'];

		$perlQF{'perlQQuestion'} = ['SID','QID','SortOrder','Question','Type','Mult','QuestionStyle','Instructions','Answer','Choose','DefaultAnswer','LargestAID','ColNumb','AnswerInputStyle','AnswerTextStyle','AnswerStyle','Attributes','Required','Summary'];
		$perlQC{'perlQQuestion'} = "CREATE TABLE perlQQuestion (SID BIGINT UNSIGNED NOT NULL, QID BIGINT UNSIGNED NOT NULL PRIMARY KEY AUTO_INCREMENT, SortOrder INT UNSIGNED NOT NULL, Question TEXT NOT NULL, Type ENUM('radio','checkbox','textfield','textarea','select','selectmult','pagebreak','htmlcontent') NOT NULL DEFAULT 'radio', Mult INT UNSIGNED NOT NULL DEFAULT 1, QuestionStyle VARCHAR(255) NOT NULL, Instructions TEXT NOT NULL, Answer TEXT NOT NULL, Choose VARCHAR(11) NOT NULL, DefaultAnswer TEXT NOT NULL, LargestAID INT UNSIGNED NOT NULL, ColNumb INT UNSIGNED NOT NULL DEFAULT 1, AnswerInputStyle VARCHAR(255) NOT NULL, AnswerTextStyle VARCHAR(255) NOT NULL, AnswerStyle VARCHAR(255) NOT NULL, Attributes TEXT NOT NULL, Required ENUM('y','n') NOT NULL DEFAULT 'y', Summary ENUM('y','n') NOT NULL DEFAULT 'n')";
		$perlQR{'perlQQuestion'} = [0, 'SID'];
		$perlQU{'perlQQuestion'} = [1, 'QID'];

		$perlQF{'perlQSurvey'} = ['SID','AddDate','Name','SurveyUsers','SurveyResults','Width','Indent','QuestionNumb','QuestionStyle','AnswerInputStyle','AnswerTextStyle','AnswerStyle','ErrorStyle','VerifyEmail','EmailFrom','EmailSubject','Template','Language','OnComplete','RedirectURL','Completed','Description','AdminNotes','Active','StartDate','EndDate','AdvanceInterface','ResultsWidth','ResultsBorder','ResultsAnswer','ResultsStats','ResultsGraph','SubmitImageSrc','SubmitImageWidth','SubmitImageHeight','SubmitImageText','ResultImageSrc','ResultImageWidth','ResultImageHeight','ResultImageText','NextImageSrc','NextImageWidth','NextImageHeight','NextImageText','ResponseNotice','ResponseEmail'];
		$perlQC{'perlQSurvey'} = "CREATE TABLE perlQSurvey (SID BIGINT UNSIGNED NOT NULL PRIMARY KEY AUTO_INCREMENT, Name VARCHAR(255) NOT NULL, AddDate DATETIME NOT NULL DEFAULT '0000-00-00', SurveyUsers CHAR(3) NOT NULL DEFAULT '000', SurveyResults TINYINT NOT NULL, Width VARCHAR(255) NOT NULL, Indent INT UNSIGNED NOT NULL DEFAULT 20, QuestionNumb ENUM('y','n') NOT NULL DEFAULT 'y', QuestionStyle VARCHAR(255) NOT NULL, AnswerInputStyle VARCHAR(255) NOT NULL, AnswerTextStyle VARCHAR(255) NOT NULL, AnswerStyle VARCHAR(255) NOT NULL, ErrorStyle VARCHAR(255) NOT NULL, VerifyEmail TEXT NOT NULL, EmailFrom VARCHAR(255) NOT NULL, EmailSubject VARCHAR(255) NOT NULL, Template VARCHAR(50) NOT NULL, Language VARCHAR(50) NOT NULL, OnComplete TINYINT NOT NULL, RedirectURL TEXT NOT NULL, Completed TEXT NOT NULL, Description TEXT NOT NULL, AdminNotes TEXT NOT NULL, Active ENUM('y','n') NOT NULL DEFAULT 'n', StartDate DATETIME NOT NULL DEFAULT '0000-00-00', EndDate DATETIME NOT NULL DEFAULT '0000-00-00', AdvanceInterface ENUM('y','n') NOT NULL DEFAULT 'n', ResultsWidth VARCHAR(255) NOT NULL, ResultsBorder VARCHAR(255) NOT NULL, ResultsAnswer VARCHAR(255) NOT NULL, ResultsStats VARCHAR(255) NOT NULL, ResultsGraph VARCHAR(255) NOT NULL, SubmitImageSrc VARCHAR(255) NOT NULL, SubmitImageWidth INT UNSIGNED NOT NULL, SubmitImageHeight INT UNSIGNED NOT NULL, SubmitImageText VARCHAR(255) NOT NULL, ResultImageSrc VARCHAR(255) NOT NULL, ResultImageWidth INT UNSIGNED NOT NULL, ResultImageHeight INT UNSIGNED NOT NULL, ResultImageText VARCHAR(255) NOT NULL,NextImageSrc VARCHAR(255) NOT NULL, NextImageWidth INT UNSIGNED NOT NULL, NextImageHeight INT UNSIGNED NOT NULL, NextImageText VARCHAR(255) NOT NULL, ResponseNotice TINYINT NOT NULL, ResponseEmail TEXT NOT NULL)";
		$perlQU{'perlQSurvey'} = [0, 'SID'];

		$perlQF{'perlQUser'} = ['UID','SID','IPAddr','BlockDupe','CheckType','Email','Confirm','EmailConfirm','CompleteDate'];
		$perlQC{'perlQUser'} = "CREATE TABLE perlQUser (UID BIGINT UNSIGNED NOT NULL PRIMARY KEY DEFAULT 0, SID BIGINT UNSIGNED NOT NULL, IPAddr VARCHAR(100) NOT NULL, BlockDupe VARCHAR(40) NOT NULL, CheckType CHAR(3) NOT NULL DEFAULT '000', Email VARCHAR(255) NOT NULL, Confirm VARCHAR(50) NOT NULL, EmailConfirm ENUM('y','n') NOT NULL DEFAULT 'n', CompleteDate DATETIME NOT NULL DEFAULT '0000-00-00')";
		$perlQR{'perlQUser'} = [1, 'SID'];
		$perlQU{'perlQUser'} = [0, 'UID'];
	}

	my %columnorder;
	for (my $l = 0; $l < @$content; $l++) {
		chomp($content->[$l]);
		my @linevals = split(/\|/, $content->[$l]);
		my $table = $linevals[1];
		if ($linevals[0] eq 'TABLE' && $linevals[2] eq 'COLUMNS') {
			if (!$perlQF{$table}) {
				$html->setMessage($action.$lang->{'ImportMissingTableColumn'});
				return 0;
			}
			my @colorder = split(/,/, $linevals[3]);
			$columnorder{$table} = \@colorder;
		}
		elsif ($linevals[0] eq 'TABLE' && $linevals[2] eq 'VALUES') {
			my @values = split(/\&/, $linevals[3]);
			my %valuenames;

			if (!$columnorder{$table}) {
				$html->setMessage($action.$lang->{'ImportMissingTableValue'});
				return 0;
			}

			# If exported version is less than 2.1, it will be missing these fields.
			# So we provide default values so the results appear like they would have.
			if ($version < 2.1 && $table == 'perlQSurvey') {
				$valuenames{'SubmitImageText'} = $lang->{'Submit'};
				$valuenames{'ResultImageText'} = $lang->{'ShowResults'};
				$valuenames{'NextImageText'} = $lang->{'Next'};
			}

			# This loop extracts the values from the lines and places them in an
			# array according to $columnorder[]
			for (my $i = 0; $i < @values; $i++) {
				my $field = $columnorder{$table}[$i];
				# If no other action is taken below, the value for this field should be:
				$valuenames{$field} = $values[$i];

				if ($table eq 'perlQSurvey') {
					# On a survey swap, we make the status inactive on import.
					if ($field eq 'Active' && $btype eq 'SURVEYSWAP') {
						$valuenames{'Active'} = 'n';
					}

					# Make sure that if a template is defined, this user has it.
					# Otherwise, we set it to the default or first available template.
					if ($field eq 'Template') {
						$valuenames{'Template'} = &findITemplate($perlQ, $values[$i]);
					}
					elsif ($field eq 'Language') {
						$valuenames{'Language'} = &findILanguage($perlQ, $values[$i]);
					}
				}
			}

			# Here we make each value SQL-safe and also order them according to our
			# own table definitions. This will exclude any fake fields.
			my @sqlvalues = ();
			for (my $i = 0; $i < @{ $perlQF{$table} }; $i++) {
				my $value = $valuenames{$perlQF{$table}->[$i]} || '';
				push(@sqlvalues, &PERLQ2SQL($perlQ, $value));
			}

			if ($btype eq 'SURVEYSWAP') {
				if ($table eq 'perlQSurvey') {
					eval {
						$dbh->do("INSERT INTO perlQSurvey (SID,AddDate," . join(',', @{ $perlQF{$table} }) . ") VALUES (NULL,NOW()," . join(',', @sqlvalues) . ")");
						$sid = $dbh->{'mysql_insertid'} || 0;
					};

					if ($@ || !$sid) {
						$html->setMessage("$action$lang->{'ImportCreateSurvey'}");
						return 0;
					}
				}
				elsif ($table eq 'perlQQuestion') {
					if (!$sid || $sid !~ /^\d+$/) {
						$html->setMessage("$action$lang->{'ImportQuestionPreSurvey'}");
						return 0;
					}

					eval {
						$dbh->do("INSERT INTO perlQQuestion (SID,QID," . join(',', @{ $perlQF{$table} }) . ") VALUES ($sid,NULL," . join(',', @sqlvalues) . ")");
					};

					if ($@) {
						$html->setMessage("$action$lang->{'ImportInsertQuestion'}");
						return 0;
					}
				}
				next;
			}

			### If we are still going, btype = 'PERLQBACKUP' or 'SURVEYBACKUP'...

			# We only mess with perlQAdmin if they have requested an erase and restore
			# from PERLQBACKUP formatted files...
			if ($table eq 'perlQAdmin') {
				if ($flag eq 'erase' && $btype eq 'PERLQBACKUP' && !$tcreated{$table}) {
					# Attempt to catch and handle any error when creating perlQAdmin...
					eval {
						local $dbh->{'RaiseError'} = 0;
						$dbh->do("DROP TABLE perlQAdmin");
					};

					eval {
						$dbh->do($perlQC{'perlQAdmin'});
					};

					if ($@) {
						$html->setMessage($action.sprintf($lang->{'ImportCreateTable'}, 'perlQAdmin'));
						return 0;
					}

					eval {
						$dbh->do("INSERT INTO perlQAdmin (Passwd," . join(',', @{ $perlQF{$table} }) . ") VALUES (" . &PERLQ2SQL($perlQ, $perlQ->getAdminField('Passwd')) . ',' . join(',', @sqlvalues) . ")");
						$tcreated{$table} = 1;
					};

					if ($@) {
						$html->setMessage($action.sprintf($lang->{'ImportInsertTable'}, 'perlQAdmin'));
						return 0;
					}
				}
			}

			# Handle any surveys that need to be inserted...
			elsif ($perlQF{$table}) {
				if ($flag eq 'erase' && !$tcreated{$table}) {
					# We do not throw an error here in case their tables just don't exist.
					eval {
						local $dbh->{'RaiseError'} = 0;
						$dbh->do("DROP TABLE $table");
					};

					eval {
						$dbh->do($perlQC{$table});
						$tcreated{$table} = 1;
					};

					if ($@) {
						$html->setMessage($action.sprintf($lang->{'ImportCreateTable'}, $table));
						return 0;
					}
				}

				# Attempt to insert the data as-is.  If there is a collision with unique
				# ids of SID, QID, UID an error will be thrown and we can attempt to
				# correct.
				if ($perlQR{$table}) {
					for (my $i = 0; $i < @{ $perlQR{$table} }; $i += 2) {
						my ($rid, $rname) = ($perlQR{$table}->[$i], $perlQR{$table}->[$i+1]);

						if ($idmap{$rname}{$sqlvalues[$rid]}) {
							$sqlvalues[$rid] = $dbh->quote($idmap{$rname}{$sqlvalues[$rid]});
						}
					}
				}

				eval {
					$dbh->do("INSERT INTO $table (" . join(',', @{ $perlQF{$table} }) . ") VALUES (" . join(',', @sqlvalues) . ")");
				};

				# If our insert failed and we weren't supposed to erase data and there
				# is a field defined to be unique, we need to attempt to insert again,
				# this time with a NULL value in the place of the unique field.
				if ($@ && $flag ne 'erase' && $perlQU{$table}) {
					my @tnames = @{ $perlQF{$table} };
					my @tvalues = @sqlvalues;
					splice(@tnames, $perlQU{$table}->[0], 1);
					splice(@tvalues, $perlQU{$table}->[0], 1);
					my $ufield = $perlQU{$table}->[1];
					my $uvalue = ($ufield eq 'UID') ? $perlQ->newUID : 'NULL';
					eval {
						$dbh->do("INSERT INTO $table ($ufield," . join(',', @tnames) . ") VALUES ($uvalue," . join(',', @tvalues) . ")");
						$uvalue = ($ufield eq 'UID') ? $uvalue : $dbh->{'mysql_insertid'};
						$idmap{$ufield}{$sqlvalues[$perlQU{$table}->[0]]} = $uvalue;
					};

					if ($@) {
						$html->setMessage($action.sprintf($lang->{'ImportInsertTable'}, $table));
						return 0;
					}
				}
				elsif ($@) {
					$html->setMessage($action.sprintf($lang->{'ImportInsertTable'}, $table));
					return 0;
				}
			}
		}
	}

	# Here we make sure that any tables that have not been dropped and recreated
	# are erased if the restore calls for it.
	if ($flag eq 'erase') {
		foreach my $table ('perlQSurvey','perlQQuestion','perlQAnswer','perlQUser') {
			if (!$tcreated{$table}) {
				# We do not throw an error here in case their tables just don't exist.
				eval {
					local $dbh->{'RaiseError'} = 0;
					$dbh->do("DROP TABLE $table");
				};

				eval {
					$dbh->do($perlQC{$table});
					$tcreated{$table} = 1;
				};

				if ($@) {
					$html->setMessage($action.sprintf($lang->{'ImportCreateTable'}, $table));
					return 0;
				}
			}
		}
	}

	return $sid;
}

sub PERLQ2SQL {
	my ($perlQ, $mystring) = @_;
	if ($mystring eq '=NULL=') { $mystring = ''; } # No NULL fields used anymore
	return $perlQ->getDBH->quote($perlQ->getHTML->urldecode($mystring));
}

sub findITemplate {
	my ($perlQ, $template) = @_;
	my %templates = $perlQ->getTemplates;

	if (!(keys %templates)) { return ''; }

	if (defined $templates{$template}) { return $template; }

	if (defined $templates{$perlQ->getAdminField('Template')}) {
		return $perlQ->getAdminField('Template');
	}

	if (defined $templates{'default'}) { return 'default'; }

	foreach my $temp (keys %templates) { return $temp; }

	return '';
}

sub findILanguage {
	my ($perlQ, $language) = @_;
	my %languages = $perlQ->getLanguages;

	if (!(keys %languages)) { return ''; }

	if (defined $languages{$language}) { return $language; }

	if (defined $languages{$perlQ->getAdminField('Language')}) {
		return $perlQ->getAdminField('Language');
	}

	if (defined $languages{'english'}) { return 'english'; }

	foreach my $lang (keys %languages) { return $lang; }

	return '';
}

1;
