#!/usr/bin/perl
################################################################################
# perlQuestionnaire                          Version 2.2                       #
# Copyright 2003-2005 chumpsoft, inc.        March 28, 2005                    #
################################################################################
#              SCRiPTMAFiA 2005 - THE DiRTY HANDS ON YOUR SCRiPTS              #
################################################################################

package perlQMod::State;
use strict;
use CGI;

sub new {
	my $this = bless {}, shift;

	my %args = (-skipdb => 0, -inadmin => 0, -redirecterror => 1, @_);
	my $skipdb = shift || 0;

	$this->setInAdmin($args{'-inadmin'});

	$this->{'_winos'} = 0;
	$this->{'_dirdel'} = '/';
	$this->{'_scriptdir'} = '';
	if ($0 =~ /(.*)\\[^\\]+/) {
		$this->{'_winos'} = 1;
		$this->{'_dirdel'} = '\\';
		$this->{'_scriptdir'} = $1.'\\';
	}

	# We must skip this step in the install.
	if (!$args{'-skipdb'}) {
		eval {
			my $dbh = $this->getDBH;

			my $query = $dbh->prepare("SELECT * FROM perlQAdmin LIMIT 1");
			if ($query->execute && $query->rows) {
				$this->{'_perlQAdmin'} = $query->fetchrow_hashref;
			}
			$query->execute;
		};

		if ($@) {
			if ($args{'-redirecterror'}) {
				my $url = $args{'-inadmin'} ? 'index.cgi' : 'admin/index.cgi';
				$this->getHTML->redirect(-url => $url);
			}
			else {
				$this->setLanguage('', 'ADMIN');
				my $lang = $this->getLanguage;
				$this->getHTML->setMessage($lang->{'DBConnection'});
			}
		}
	}

	return $this;
}

sub getVersion { return '2.2'; }
sub inAdmin { my $this = shift; return $this->{'_inadmin'}; }
sub setInAdmin { my $this = shift; $this->{'_inadmin'} = shift || 0; }

sub setImageDir { my $this = shift; $this->{'_limgdir'} = shift || ''; }
sub getImageDir { my $this = shift; return $this->getImgURL($this->{'_limgdir'} || '', shift); }
sub getImgURL {
	my $this = shift; 
	my $path = shift;
	my $force = shift;
	# This requires $force to be defined and true or undefined and admin field
	# set. Therefore, if it is defined and false, it will bypass this.
	# also, if $force is not defined and we could not retrieve anything from admin
	# we default to using img.cgi in case it hasn't been installed yet...
	if (((defined $force) && $force) ||
	    ((!defined $force) && ((!defined $this->{'_perlQAdmin'}) || $this->getAdminField('InCgiBin') eq 'y'))) {
		$path = 'img.cgi?i=' . $path;
	}

	if ($this->inAdmin) {
		return '../' . $path;
	}
	return $path;
}

sub getAdminField {
	my $this = shift;
	my $field = shift;

	if (!defined $this->{'_perlQAdmin'}) { return ''; }

	if (defined $this->{'_perlQAdmin'}{$field}) {
		return $this->{'_perlQAdmin'}{$field};
	}
	return '';
}

sub setPageNumber { my $this = shift; $this->{'_pagenumber'} = shift || ''; }
sub getPageNumber { my $this = shift; return $this->{'_pagenumber'} || ''; }

sub getDBH {
	my $this = shift;
	if (!$this->{'_dbh'}) {
		eval {
			require "DBInfo.cgi";
			import perlQMod::DBInfo;

			$this->{'_dbh'} = perlQMod::DBInfo->connect(@_);
		};

		if ($@) {
			$this->{'_dbh'} = 0;
		}
	}

	return $this->{'_dbh'};
}

sub getCGI {
	my $this = shift;
	if (!$this->{'_form'}) {
		$this->{'_form'} = CGI->new;
	}

	return $this->{'_form'};
}

sub getUploadFH {
	my $this = shift;
	my $field = shift;

	if ($CGI::VERSION >= 2.47) {
		return $this->getCGI->upload($field); 
	}
	return $this->getCGI->param($field);
}

sub getHTML {
	my $this = shift;

	if (!$this->{'_html'}) {
		$this->includeMod('HTML');
		$this->{'_html'} = perlQMod::HTML->new($this);
	}

	return $this->{'_html'};
}

sub getFilePath {
	my $this = shift;
	my $distropath = shift;
	if ($this->{'_winos'}) { $distropath =~ s/\//\\/g; }

	my $dir = $this->{'_scriptdir'};
	$dir .= $this->inAdmin ? "..$this->{'_dirdel'}" : '';
	return $dir . $distropath;
}
	

sub sidValid {
	my $this = shift;
	my $sid = shift;
	my $dbh = $this->getDBH;
	my $retval = 0;

	if ($sid !~ /^\d+$/) { return 0; }

	my $query = $dbh->prepare("SELECT 1 FROM perlQSurvey WHERE SID=?");
	if ($query->execute($sid) && $query->rows) { $retval = 1; }
	$query->finish;
	return $retval;
}

sub qidValid {
	my $this = shift;
	my $qid = shift;
	my $dbh = $this->getDBH;
	my $retval = 0;

	if ($qid !~ /^\d+$/) { return 0; }

	my $query = $dbh->prepare("SELECT 1 FROM perlQQuestion WHERE QID=?");
	if ($query->execute($qid) && $query->rows) { $retval = 1; }
	$query->finish;
	return $retval;
}

# Months start with jan=1
sub dateValid {
	my $this = shift;
	my ($sec,$min,$hour,$mday,$mon,$year) = @_;
	my @maxmdays = (0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
	if ($year % 4 == 0 && ($year % 100 != 0 || $year % 400 == 0)) {
		$maxmdays[2] = 29;
	}

	if ($mon < 1 || $mon > 12) { return 0; }
	if ($mday < 1 || $mday > $maxmdays[$mon]) { return 0; }
	if ($sec < 0 || $sec > 59) { return 0; }
	if ($min < 0 || $min > 59) { return 0; }
	if ($hour < 0 || $hour > 23) { return 0; }
	return 1;
}

sub newUID {
	my $this = shift;
	return $this->includeMod('GenerateUID') ? perlQMod::GenerateUID->new($this->getDBH) : 0;
}

sub includeMod {
	my $this = shift;
	my $mod = shift;
	if (!$this->{'_mods'}{$mod}) {
		eval {
			require "$mod\.cgi";
			my $modfile = "perlQMod::$mod";
			import $modfile;
			$this->{'_mods'}{$mod} = 1;
		};

		if ($@) { die $@; return 0; }
	}

	return 1;
}

sub login {
	my $this = shift;
	my $dbh = $this->getDBH;
	my $html = $this->getHTML;

	my $passwd = $this->getCGI->param('perlQpasswd');
	if (!$passwd) { return 0; }

	my $query = $dbh->prepare("SELECT 1 FROM perlQAdmin WHERE Passwd=? LIMIT 1");
	if ($query->execute($passwd) && $query->rows) {
		my @sesschars = ('A'..'Z','a'..'z','0'..'9');
		my $session_id = '';
		for (my $i = 0; $i < 25; $i++) {
			$session_id .= $sesschars[int(rand($#sesschars))];
		}

		eval {
			$dbh->do("INSERT INTO perlQSession (SessionID,RecentTime) VALUES (?,?)", undef, $session_id, time);
			$query->finish;
			$html->setcookie(-name => 'perlQSession', -value => $session_id, -path => '/');
			$html->redirect(-url => 'main.cgi');
		};

		if ($@) {
			$html->setMessage('Database Error when Logging In!');
		}
	}
	else {
		$html->setMessage('Invalid Password!');
	}
	$query->finish;

	return 0;
}

sub logout {
	my $this = shift;
	my $dbh = $this->getDBH;
	my $html = $this->getHTML;

	my $session_id = $this->getCGI->cookie('perlQSession');
	if (!$session_id) { return 1; }

	eval {
		$dbh->do("DELETE FROM perlQSession WHERE SessionID=?", undef, $session_id);
	};

	$html->setcookie(-name => 'perlQSession', -value => '', -path => '/');
	$html->redirect(-url => 'index.cgi');
}

sub isAdmin {
	my $this = shift;
	my %args = (-ispopup => 0, -redirecterrors => 0, @_);
	my $ispopup = shift;
	my $cgi = $this->getCGI;
	my $html = $this->getHTML;

	my $session_id = $cgi->cookie('perlQSession');
	if (!$session_id) {
		if ($args{'-redirecterrors'}) {
			if ($args{'-ispopup'}) {
				$html->redirect(-url => 'popuperror.cgi?errormsg=PopupLogin');
			}
			else {
				$html->redirect(-url => 'index.cgi');
			}
		}
		else { return 0; }
	}

	my $timeout = $this->getAdminField('TimeOut');
	if (!$timeout || $timeout < 60) { $timeout = 86400; }

	my $dbh = $this->getDBH;
	my $query = $dbh->prepare("SELECT * FROM perlQSession WHERE SessionID=?");
	if ($query->execute($session_id) && $query->rows) {
		my $masession = $query->fetchrow_hashref;
		if ((time - $masession->{'RecentTime'}) < $timeout) {
			eval {
				$dbh->do("DELETE FROM perlQSession WHERE (? - RecentTime) > ? AND SessionID != ?", undef, time, $timeout, $session_id);
				$dbh->do("UPDATE perlQSession SET RecentTime=? WHERE SessionID=?", undef, time, $session_id);
			};

			if (!$@) {
				$query->finish;
				return 1;
			}
		}
	}
	$query->finish;

	if ($args{'-redirecterrors'}) {
		if ($args{'-ispopup'}) {
			$html->redirect(-url => 'popuperror.cgi?errormsg=PopupLoggedOut');
		}
		else {
			$html->redirect(-url => 'index.cgi?errormsg=LoggedOut');
		}
	}

	return 0;
}

sub setLanguage {
	my $this = shift;
	my $language = shift || '';
	my $type = shift || 'PUBLIC';

	if ($this->requireLanguage($language)) { return $language; }

	if ($type eq 'PUBLIC') { $language = $this->getAdminField('Language'); }
	else { $language = $this->getAdminField('AdminLanguage'); }
	if ($this->requireLanguage($language)) { return $language; }

	if ($this->requireLanguage('english')) { return 'english'; }

	my %languages = $this->getLanguages($type);
	if (%languages) {
		foreach my $language (sort { $a cmp $b } keys %languages) {
			if ($this->requireLanguage($language)) { return $language; }
		}
	}

	$this->getCGI->header;
	print "No Language File Found!";
	$this->exit;
}

sub requireLanguage {
	my $this = shift;
	my $language = shift;

	my $langdir = $this->getFilePath('languages/');

	my $langfile = "$langdir$language$this->{'_dirdel'}index.pl";

	if ($language ne '' && -r $langfile) {
		use vars qw(%perlQlang);
		%perlQlang = ();
		do "$langfile";
		$this->{'_language'} = \%perlQlang;

		# Find the appropriate image URL to use if we're in admin...
		my %imglangs = $this->getLanguages('IMAGES');
		my $imglang = '';
		if ($imglangs{$language} && -d "$langdir$language$this->{'_dirdel'}images") {
			$imglang = $language;
		}
		elsif ($imglangs{'english'} && -d $langdir."english$this->{'_dirdel'}images") {
			$imglang = 'english';
		}
		else {
			foreach my $lang (sort { $a cmp $b } keys %imglangs) {
				if (-d "$langdir$lang$this->{'_dirdel'}images") { $imglang = $lang; last; }
			}
		}

		if ($imglang ne '') { $this->setImageDir("languages/$imglang/images"); }

		return 1;
	}

	return 0;
}

sub getLanguage { my $this = shift; return $this->{'_language'}; }

sub getLanguages {
	my $this = shift;
	my $type = shift || 'PUBLIC';

	my %languages = ();

	my $languagesdir = $this->getFilePath('languages/');

	if (opendir(LDIR, "$languagesdir")) {
		my @langs = readdir(LDIR);
		foreach my $lang (@langs) {
			if ($lang =~ /^\./ || !(-d "$languagesdir$lang")) { next; }
			my $languageloc = "$languagesdir$this->{'_dirdel'}$lang$this->{'_dirdel'}index.pl";
			if (-r $languageloc) {
				open(LANG, "$languageloc");
				my $header = <LANG>;
				close(LANG);

				my @languageinfo = split(/:/, $header);
				for (my $i = 0; $i < @languageinfo; $i++) {
					$languageinfo[$i] =~ s/^\s+//;
					$languageinfo[$i] =~ s/\s+$//;
				}
				my %languagetype = map { $_ => 1 } split(/\|/,$languageinfo[2]);

				if (%languagetype && $languagetype{$type}) {
					$languages{$lang} = ($languageinfo[1] ne '') ? $languageinfo[1] : $lang;
				}
			}
		}
		closedir(LDIR);
	}

	return %languages;
}

sub getTemplates {
	my $this = shift;
	my %templates = ();

	my $templatesdir = $this->getFilePath('templates/');

	if (opendir(TDIR, "$templatesdir")) {
		my @temps = readdir(TDIR);
		foreach my $temp (@temps) {
			if ($temp =~ /^\./ || !(-d "$templatesdir$temp")) { next; }
			my $templateloc = "$templatesdir$temp$this->{'_dirdel'}index.html";
			if (-r $templateloc) {
				open(TEMP, "$templateloc");
				my $header = <TEMP>;
				close(TEMP);

				my $templatename = $temp;
				if ($header =~ /<!--\/\/(.*)\/\/-->/) {
					my @templateinfo = split(/:/, $1);
					$templateinfo[1] =~ s/^\s+//;
					$templateinfo[1] =~ s/\s+$//;
					if ($templateinfo[1] ne '') { $templatename = $templateinfo[1]; }
				}
				$templates{$temp} = $templatename;
			}
		}
		close(TDIR);
	}

	return %templates;
}

sub sendEmail {
	my $this = shift;
	my %args = (-headers => '', @_);
	my $dbh = $this->getDBH;

	my ($sendmailpath, $smtpserver) = ('', '');
	my $query = $dbh->prepare("SELECT SendmailPath, SMTPServer FROM perlQAdmin LIMIT 1");
	if ($query->execute && $query->rows) {
		($sendmailpath, $smtpserver) = $query->fetchrow_array;
	}
	$query->finish;

	my $sent = 0;
	if ($sendmailpath && -x $sendmailpath) {
		if (open(MAIL, "|$sendmailpath -t -oi")) {
			print MAIL "From: $args{'-from'}\n";
			print MAIL "To: $args{'-to'}\n";
			print MAIL "Subject: $args{'-subject'}\n";
			if ($args{'-headers'} ne '') { print MAIL $args{'-headers'}; }
			print MAIL "\n";
			print MAIL $args{'-body'};
			close(MAIL);
			$sent = 1;
		}
	}

	if (!$sent && $smtpserver) {
		eval {
			require Net::SMTP;
			import Net::SMTP;

			my $smtp = Net::SMTP->new($smtpserver);
			if ($smtp) {
				$smtp->mail($this->parseEmailAddress($args{'-from'}));
				$smtp->to($this->parseEmailAddress($args{'-to'}));

				$smtp->data;
				$smtp->datasend("From: $args{'-from'}\n");
				$smtp->datasend("To: $args{'-to'}\n");
				$smtp->datasend("Subject: $args{'-subject'}\n");
				if ($args{'-headers'} ne '') { $smtp->datasend($args{'-headers'}); }
				$smtp->datasend("\n");
				$smtp->datasend($args{'-body'});
				$smtp->dataend;

				$smtp->quit;
				$sent = 1;
			}
		};

		if ($@) {
			print "Content-type: text/html\n\nerror: $@\n";
		}
	}

	return $sent;
}

sub parseEmailAddress {
	my $this = shift;
	my $email = shift;

	if ($email =~ /\<([^>\s]+\@[^>\s]+\.[^>\s]+)\>/) { return $1; }
	elsif ($email =~ /(\S+\@\S+\.\S+)/) { return $1; }
	else { return $email; }
}

sub exit {
	my $this = shift;
	if ($this->{'_dbh'}) { $this->{'_dbh'}->disconnect; }
	exit;
}

1;
