#!/usr/bin/perl
################################################################################
# perlQuestionnaire                          Version 2.2                       #
# Copyright 2003-2005 chumpsoft, inc.        March 28, 2005                    #
################################################################################
#              SCRiPTMAFiA 2005 - THE DiRTY HANDS ON YOUR SCRiPTS              #
################################################################################

#-->> Push correct path into @INC depending on whether this is Unix/Windows.
if ($0 =~ /(.*)\\[^\\]+/) { unshift (@INC, "$1\\perlQMod"); }
elsif ($0 =~ /(.*)\/[^\/]+/) { unshift (@INC, "$1/perlQMod"); }
else { unshift (@INC, "perlQMod"); }

#-->> Include Files

use strict;
require "State.cgi";
import perlQMod::State;
require "Question.cgi";

my $perlQ = perlQMod::State->new;
my $cgi = $perlQ->getCGI;
my $dbh = $perlQ->getDBH;
my $html = $perlQ->getHTML;

#-->> Setting Variables

my $sid = (($cgi->param('sid') || 0) =~ /^(\d+)$/) ? $1 : 0;
my $uid = (($cgi->param('uid') || 0) =~ /^(\d+)$/) ? $1 : 0;
my $emailaddr = $cgi->param('emailaddr') || '';
my $vsubmit = $cgi->param('vsubmit') || 0;

#-->> Check UID and SID

if (!$perlQ->getAdminField('SendmailPath') && !$perlQ->getAdminField('SMTPServer')) {
	$html->redirect(-url => 'error.cgi?errormsg=NoMailSupport');
}

my ($masurvey,$mauser) = 0;
my $query = $dbh->prepare("SELECT * FROM perlQSurvey WHERE SID=?");
if ($query->execute($sid) && $query->rows) {
	$masurvey = $query->fetchrow_hashref;
}
$query->finish;

$query = $dbh->prepare("SELECT * FROM perlQUser WHERE SID=? AND UID=?");
if ($query->execute($sid, $uid) && $query->rows) {
	$mauser = $query->fetchrow_hashref;
}
$query->finish;

if (!$masurvey || !$mauser) { $html->redirect(-url => 'error.cgi'); }

# If an email address is already attached to this survey response, just forward
# them to the confirm page...
my $email = '';
$query = $dbh->prepare("SELECT Email FROM perlQUser WHERE SID=? AND UID=?");
if ($query->execute($sid, $uid) && $query->rows) {
	$email = ($query->fetchrow_array)[0];
}
$query->finish;
if ($email) { $html->redirect(-url => "confirm.cgi?uid=$uid&sid=$sid"); }

#-->> Set Language

$perlQ->setLanguage($masurvey->{'Language'});
my $lang = $perlQ->getLanguage;

#-->> Create Form

my $errormsg = '';
my $errorstyle = $masurvey->{'ErrorStyle'} || 'color: red;';
if ($emailaddr) {
	my $dbemail = '';
	$query = $dbh->prepare("SELECT Email FROM perlQUser WHERE UID=? AND SID=?");
	if ($query->execute($uid, $sid) && $query->rows) {
		$dbemail = ($query->fetchrow_array)[0];
	}
	$query->finish;

	# If an email address is already recorded, go to confirm.
	if ($dbemail ne '') {
		$html->redirect(-url => "confirm.cgi?uid=$uid&sid=$sid");
	}

	if ($emailaddr !~ /.+\@.+\..+/ || $emailaddr =~ /(\n|\r)/) {
		$errormsg = sprintf($lang->{'EmailInvalid'}, $html->protectHTML($emailaddr));
	}
	else {
		$query = $dbh->prepare("SELECT 1 FROM perlQUser WHERE Email=? AND SID=?");
		if ($query->execute($emailaddr, $sid) && $query->rows) {
			$errormsg = sprintf($lang->{'EmailDuplicate'}, $html->protectHTML($emailaddr));
		}
		$query->finish;
	}

	if (!$errormsg) {
		my $emailmsg = '';
		my $scriptloc = $cgi->url;
		if ($scriptloc =~ m#^(.*)/[^/]+$#) { $scriptloc = $1; }
		my $confirmurl = "$scriptloc/confirm.cgi?uid=$uid&sid=$sid&upasswd=$mauser->{'Confirm'}";

		if ($masurvey->{'VerifyEmail'}) { $emailmsg = $masurvey->{'VerifyEmail'}; }
		else { $emailmsg = $lang->{'EmailDefaultMessage'}; }

		$emailmsg =~ s/::SurveyName::/$masurvey->{'Name'}/g;
		$emailmsg =~ s/::ConfirmPassword::/$mauser->{'Confirm'}/g;
		$emailmsg =~ s/::ConfirmURL::/$confirmurl/g;

		my $from = $masurvey->{'EmailFrom'} || $lang->{'EmailDefaultFrom'};
		my $subject = $masurvey->{'EmailSubject'} || sprintf($lang->{'EmailDefaultSubject'}, $masurvey->{'Name'});
		
		$perlQ->sendEmail(-to => $emailaddr, -subject => $subject, -body => $emailmsg, -from => $from);

		eval {
			$dbh->do("UPDATE perlQUser SET Email=? WHERE SID=? AND UID=?", undef, $emailaddr, $sid, $uid);
		};

		$html->redirect(-url => "confirm.cgi?uid=$uid&sid=$sid");
	}
}

if (!$errormsg && $vsubmit) { $errormsg = $lang->{'EmailEmpty'}; }

my $button = &getButton($perlQ, $masurvey, 'submit');
my $contents = "<TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=$masurvey->{'Width'}><TR><TD ALIGN=center><TABLE CELLSPACING=0 CELLPADDING=2 BORDER=0>\n";
$contents .= "<TR><TD>$lang->{'VerifyRequest'}</TD></TR>\n";
$contents .= "</TABLE>\n";
$contents .= "<TABLE CELLPADDING=2 CELLSPACING=0 BORDER=0>\n";
$contents .= "<FORM ACTION=verify.cgi METHOD=POST>\n";
$contents .= "<INPUT TYPE=hidden NAME=sid VALUE=$sid>\n";
$contents .= "<INPUT TYPE=hidden NAME=uid VALUE=$uid>\n";
$contents .= "<INPUT TYPE=hidden NAME=vsubmit VALUE=1>\n";
if ($errormsg) {
	$contents .= "<TR><TD COLSPAN=3 STYLE=\"$errorstyle\">$errormsg</TD></TR>\n";
}
$contents .= "<TR>\n";
$contents .= "<TD><B>$lang->{'EmailAddress'}:</B></TD>\n";
$contents .= "<TD><INPUT TYPE=text NAME=emailaddr VALUE=\"" . $html->protectHTML($emailaddr) . "\"></TD>\n";
$contents .= "<TD>$button</TD>\n";
$contents .= "</TR>\n";
$contents .= "</FORM>\n";
$contents .= "</TABLE></TD></TR></TABLE>\n";

#-->> Output Page

my $title = sprintf($lang->{'VerifyTitle'}, $masurvey->{'Name'});
$html->createPage($sid, $title, $contents, 0, '');
